#include "DTSource.h"
#include "DTSaveError.h"

extern int ComputeTask(void);

int main(int argc,const char *argv[])
{
    DTSetArguments(argc,argv);
	
    return ComputeTask();
}

int CountTimeValues(const DTDataStorage &inputFile,string variable)
{
    // Check for the existence of variable_#.  Again, this can be made
    // much more efficient with a binary type search, but do a linear here.
    int toReturn = 0;
    while (inputFile.Contains(variable+"_"+DTInt2String(toReturn)))
        toReturn++;
    if (toReturn==0 && inputFile.Contains(variable))
        toReturn++;
    return toReturn;
}

void DisplayGroup(const DTDataStorage &inputFile,string variable,string basename,string padding)
{
    cerr << padding << variable << " - Group";
    
    if (padding.length()==0) {
        // Top level, check how many time values are saved.
        int howManyTimeValues = CountTimeValues(inputFile,variable);
        if (howManyTimeValues>1) {
            cerr << " - " << howManyTimeValues << " time values";
        }
    }
    cerr << endl;
        
    padding = padding + "    ";
    
	// Now go over each sub entry
	int howMany = inputFile.ReadInt(basename+"_N");
	int i;
    string type;
    for (i=0;i<howMany;i++) {
        variable = inputFile.ReadString(basename+"_"+DTInt2String(i+1)+"N");
        type = inputFile.ReadString(basename+"_"+DTInt2String(i+1));
        if (type=="Group")
            DisplayGroup(inputFile,variable,basename+"_"+DTInt2String(i+1),padding);
        else
            cerr << padding << variable << " - " << type << endl;
    }
}

void PrintReport(const DTDataStorage &inputFile)
{
    DTList<string> variables = inputFile.AllVariableNames();
    
	// Find all the compartments
	variables = inputFile.AllVariableNamesWithPrefix("Seq_");
    
	int howMany = variables.Length();
	string variable,basename;
    string type;
    int i,howManyTimeValues;
	for (i=0;i<howMany;i++) {
		basename = variables(i);
        type = inputFile.ReadString(basename);
		variable = basename.substr(4,basename.length());
        if (type=="Group")
            DisplayGroup(inputFile,variable,"SeqInfo_"+variable,"");
        else {
            howManyTimeValues = CountTimeValues(inputFile,variable);
            if (howManyTimeValues>1) {
                cerr << variable << " - " << type << " - " << howManyTimeValues << " time values" << endl;
            }
            else {
                cerr << variable << " - " << type << endl;
            }
        }
	}
}    

int ComputeTask(void)
{
    string fileName = "../Data Files/Variables.mat";
    
    // Find if there is a file argument
	int i;
    int howMany = DTHowManyArguments();
    string flagEntry;
    for (i=0;i<howMany;i++) {
        flagEntry = DTGetArgumentNumber(i);
        if (flagEntry[0]!='-') {
            fileName = flagEntry;
            break;
        }
    }
    
    // Check if this is a matlab file
    
    DTMatlabDataFile inputFile(fileName,DTFile::ReadOnly);
    if (!inputFile.IsOpen()) {
        cerr << "Could not open the file " << fileName << endl;
        return 0;
    }
    
    if (DTArgumentIncludesFlag("all")) {
        inputFile.pinfo();
        return 0;
    }
    
    PrintReport(inputFile);
	
    return 0;
}

