#include "DTSource.h"
#include "DTSaveError.h"
#include "DTDataContainer.h"
#include "DTLiveUpdate.h"

extern int ComputeTask(void);

int main(int argc,const char *argv[])
{
    DTSetArguments(argc,argv);
    
    string first = "Ice";
    string second = "Tuna";
    if (DTArgumentIncludesFlag("First"))
        first = DTArgumentFlagEntry("First");
    if (DTArgumentIncludesFlag("Second"))
        second = DTArgumentFlagEntry("Second");
    int publishPort = 22462;
    if (DTArgumentIncludesFlag("Port"))
        publishPort = DTArgumentIntegerEntry("Port");
    int howManyIterations = 1000;
    if (DTArgumentIncludesFlag("Iterations"))
        howManyIterations = DTArgumentIntegerEntry("Iterations");
    
    if (DTArgumentIncludesFlag("Port")) {
        while (DTLivePublish(publishPort,first,second)==false) {
            cerr << publishPort << " failed.\n";
            publishPort++;
        }
        cerr << "port = " << publishPort << endl;
    }
    else {
        DTLivePublish("/tmp/liveSocket");
        cerr << "connect through /tmp/liveSocket" << endl;
    }
    
    // Define the variables.  Can happen at any point.
	DTLiveAddVariable("A","Array");
	DTLiveAddVariable("Another","Real Number");
	DTLiveAddVariable("B_First","Real Number");
	DTLiveAddVariable("B_SubGroup","Group");
	DTLiveAddVariable("B_SubGroup_Value","Real Number");
	DTLiveAddVariable("B","Group");
	DTLiveAddVariable("B_Second","Real Number");
	
	DTMutableDoubleArray test;
    DTDataContainer dc = DTLiveDataObject(); // Will not change.
	
	DTMesh2D mesh;
	
	int N = 600;
	
	int i = 0;
	test = DTMutableDoubleArray(N,N);
	while (i<howManyIterations) {
		test = i;
		test((10+i)%100,(20+2*i)%N) = test((10+i+1)%100,(20+2*i)%N) = 0;
		test((10+i)%100,(20+2*i+1)%N) = test((10+i+1)%100,(20+2*i+1)%N) = 0;
		i++;
		if (DTLiveCurrentlySendingOver("A"))
			dc.Save(test.Copy(),"A");
		
		if (i==300) DTLiveAddVariable("A Mesh","2D Mesh"); // Late definition
        
		if (i>=300 && DTLiveCurrentlySendingOver("A Mesh")) {
			mesh = DTMesh2D(test.Copy(),DTMesh2DGrid(DTPoint2D(0,0),1.0/double(N-1)));
            WriteToFile(dc,string("A Mesh"),mesh);
		}
		
		if (DTLiveCurrentlySendingOver("Another")) 
            dc.Save(double(i),"Another");
		
		dc.Save(double(i),"B_First");
		dc.Save(double(i+3),"B_Second");
		dc.Save(double(i+6),"B_SubGroup_Value");
		
		if (i%50==0) cerr << "i = " << i << endl;
		
		if (DTLiveCurrentlyConnected()) DTLiveSynchronize();
	}

    return 0;
}

