#include "DTSource.h"

#include "DTSeriesMesh2D.h"


void Computation(void);

int main(int argc, const char * argv[])
{
    DTSetArguments(argc,argv);
    
    Computation();
    
    return 0;
}

void Computation(void)
{
    string dirName = "/tmp/SOR";
    if (DTArgumentIncludesFlag("CurrentDirectory"))
        dirName = "";
    
    DTMatlabDataFile input(dirName+"Arguments.mat",DTFile::ReadOnly);
    DTMatlabDataFile output(dirName+"Output.mat",DTFile::NewReadWrite);
    
    DTSeriesMesh2D meshOutput(output,"u");
    
    DTMesh2D u,f;
    ReadFromFile(input,"Initial Guess",u);
    ReadFromFile(input,"f",f);
    
    int howMany = input.ReadInt("How Many");
    int saveEvery = input.ReadInt("Between Saves");
    double omega = input.ReadNumber("Omega");
    
    double oneMOmega = 1-omega;
    double omega4 = omega*0.25;
    double h = u.Grid().dx();
    double hSq = h*h;
    
    // Make sure that the mesh is double precision.
    // All actions that can cause memory copies are explicit.  If the mesh
    // is double precision, no copy is performed.
    u = ConvertToDouble(u);
    f = ConvertToDouble(f);
    
    DTMutableDoubleArray uArr = u.DoubleData().Copy();
    DTDoubleArray fArr = f.DoubleData();
    double *uD = uArr.Pointer();
    const double *fD = fArr.Pointer();
    int ij;
    
    // Use this as the initial guess.
    int i,j,x,y;
    const int m = u.m();
    const int n = u.n();
    
    meshOutput.Add(u,0.0); // Initial value.
    
    // Quick implementation of a SOR iteration scheme.
    for (i=0;i<howMany;i++) {
        for (j=0;j<saveEvery;j++) {
            for (y=1;y<n-1;y++) {
                ij = (1+y%2) + m*y;
                for (x=1+y%2;x<m-1;x+=2) {
                    uD[ij] = oneMOmega*uD[ij] + omega4*(uD[ij+1] + uD[ij-1] +
                                                        uD[ij+m] + uD[ij-m] - hSq*fD[ij]);
                    ij+=2;
                }
            }
            
            for (y=1;y<n-1;y++) {
                ij = (1+(y-1)%2) + m*y;
                for (x=1+(y-1)%2;x<m-1;x+=2) {
                    uD[ij] = oneMOmega*uD[ij] + omega4*(uD[ij+1] + uD[ij-1] +
                                                        uD[ij+m] + uD[ij-m] - hSq*fD[ij]);
                    ij+=2;
                }
            }
        }
        
        // Save this
        meshOutput.Add(uArr,u.Grid(),(i+1)*saveEvery);
        cerr << "Iteration = " << i+1 << endl;
    }
}
