#include "DTSource.h"
#include "DTSaveError.h"

DTDoubleArray Computation(DTDoubleArray A);

int main(int argc,const char *argv[])
{
    DTSetArguments(argc,argv);

    DTDataFile inputFile("Input",DTFile::ReadOnly);
    // Read in the input variables.
    DTDoubleArray A = inputFile.ReadDoubleArray("A");

    // The computation.
    DTDoubleArray computed;
    clock_t t_before = clock();
    computed = Computation(A);
    clock_t t_after = clock();
    double exec_time = double(t_after-t_before)/double(CLOCKS_PER_SEC);

    // Write the output.
    DTDataFile outputFile("Output",DTFile::NewReadWrite);

    // Output from computation
    outputFile.Save(computed,"Var");
    outputFile.Save("NumberList","Seq_Var");

    // The execution time.
    outputFile.Save(exec_time,"ExecutionTime");
    outputFile.Save("Real Number","Seq_ExecutionTime");

    // The errors.
    DTSaveError(outputFile,"ExecutionErrors");
    outputFile.Save("StringList","Seq_ExecutionErrors");

    return 0;
}

DTDoubleArray Computation(DTDoubleArray A)
{
    int m = A.m();
    int n = A.n();
    
    DTMutableDoubleArray toReturn(m);

    int i,j;
    double theSum;
    for (i=0;i<m;i++) {
        theSum = 0.0;
        for (j=0;j<n;j++) {
            theSum += A(i,j);
        }
        toReturn(i) = theSum;
    }

    return toReturn;
}
