#include "DTSource.h"
#include "DTSaveError.h"

DTDoubleArray Computation(DTFile file,DTRegion1D region);

int main(int argc,const char *argv[])
{
    DTSetArguments(argc,argv);

    DTDataFile inputFile("Input",DTFile::ReadOnly);
    // Read in the input variables.
    DTFile file("file",DTFile::ReadOnly);
    DTRegion1D region;
    ReadFromFile(inputFile,"region",region);

    // The computation.
    DTDoubleArray computed;
    clock_t t_before = clock();
    computed = Computation(file,region);
    clock_t t_after = clock();
    double exec_time = double(t_after-t_before)/double(CLOCKS_PER_SEC);

    // Write the output.
    DTDataFile outputFile("Output",DTFile::NewReadWrite);

    // Output from computation
    outputFile.Save(computed,"Var");
    outputFile.Save("NumberList","Seq_Var");

    // The execution time.
    outputFile.Save(exec_time,"ExecutionTime");
    outputFile.Save("Real Number","Seq_ExecutionTime");

    // The errors.
    DTSaveError(outputFile,"ExecutionErrors");
    outputFile.Save("StringList","Seq_ExecutionErrors");

    return 0;
}

DTDoubleArray Computation(DTFile file,DTRegion1D region)
{
    int startAt = int(floor(region.minV+0.5));
    if (startAt<0) startAt = 0;
    
    int endAt = int(floor(region.maxV+0.5));
    int howManyNumbers = (endAt-startAt)/8;
    
    if (howManyNumbers<0)
        return DTDoubleArray();
    
    file.SetPosition(startAt);
    DTMutableDoubleArray toReturn(howManyNumbers);
    file.ReadBinary(toReturn);

    return toReturn;
}
